package com.fjhx.sale.service.after.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.sale.entity.after.dto.AfterSalesDto;
import com.fjhx.sale.entity.after.dto.AfterSalesSelectDto;
import com.fjhx.sale.entity.after.po.AfterSales;
import com.fjhx.sale.entity.after.po.AfterSalesDetail;
import com.fjhx.sale.entity.after.po.AfterSalesRecords;
import com.fjhx.sale.entity.after.vo.AfterSalesDetailVo;
import com.fjhx.sale.entity.after.vo.AfterSalesRecordsVo;
import com.fjhx.sale.entity.after.vo.AfterSalesVo;
import com.fjhx.sale.mapper.after.AfterSalesMapper;
import com.fjhx.sale.service.after.AfterSalesDetailService;
import com.fjhx.sale.service.after.AfterSalesRecordsService;
import com.fjhx.sale.service.after.AfterSalesService;
import com.fjhx.sale.service.contract.ContractProductService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 售后管理 服务实现类
 * </p>
 *
 * @author
 * @since 2024-04-07
 */
@Service
public class AfterSalesServiceImpl extends ServiceImpl<AfterSalesMapper, AfterSales> implements AfterSalesService {

    @Autowired
    private ContractProductService contractProductService;
    @Autowired
    private AfterSalesDetailService afterSalesDetailService;
    @Autowired
    private FlowExampleService flowExampleService;
    @Autowired
    private AfterSalesRecordsService afterSalesRecordsService;

    @Override
    public Page<AfterSalesVo> getPage(AfterSalesSelectDto dto) {
        IWrapper<AfterSales> wrapper = getWrapper();

        wrapper.keyword(dto.getKeyword(), new SqlField("c.code"));

        wrapper.eq("as1", AfterSales::getStatus, dto.getStatus());
        wrapper.eq("as1", AfterSales::getType, dto.getType());

        wrapper.orderByDesc("as1", AfterSales::getId);
        Page<AfterSalesVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<AfterSalesVo> records = page.getRecords();
        setInfo(records);
        return page;
    }

    private void setInfo(List<AfterSalesVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        List<Long> asIds = records.stream().map(AfterSales::getId).collect(Collectors.toList());
        //赋值跟进记录
        List<AfterSalesRecordsVo> afterSalesRecordList = afterSalesRecordsService.getList(IWrapper.getWrapper().in(AfterSalesRecords::getAfterSalesId, asIds));
        Map<Long, List<AfterSalesRecordsVo>> collect = afterSalesRecordList.stream().collect(Collectors.groupingBy(AfterSalesRecords::getAfterSalesId));
        for (AfterSalesVo record : records) {
            record.setAfterSalesRecordsList(collect.getOrDefault(record.getId(), new ArrayList<>()));
        }
    }

    private void setDetail(List<AfterSalesVo> records) {
        if (ObjectUtil.isEmpty(records)) {
            return;
        }
        List<Long> asIds = records.stream().map(AfterSales::getId).collect(Collectors.toList());
        //赋值明细
        List<AfterSalesDetailVo> afterSalesDetailVos = afterSalesDetailService.getList(IWrapper.getWrapper().in("asd", AfterSalesDetail::getAfterSalesId, asIds));
        Map<Long, List<AfterSalesDetailVo>> collect = afterSalesDetailVos.stream().collect(Collectors.groupingBy(AfterSalesDetail::getAfterSalesId));
        for (AfterSalesVo record : records) {
            record.setAfterSalesDetailList(collect.getOrDefault(record.getId(), new ArrayList<>()));
        }
    }

    @Override
    public AfterSalesVo detail(Long id) {
        AfterSalesVo result = baseMapper.detail(IWrapper.getWrapper().eq("as1", AfterSales::getId, id));

        setInfo(Arrays.asList(result));
        setDetail(Arrays.asList(result));
        return result;
    }

    @DSTransactional
    @Override
    public void addOrEdit(AfterSalesDto dto) {
        List<AfterSalesDetail> afterSalesDetailList = dto.getAfterSalesDetailList();
        contractProductService.attributeAssign(afterSalesDetailList, AfterSalesDetail::getContractProductId, (item, contractProduct) -> {
            item.setContractId(contractProduct.getContractId());
        });

        this.saveOrUpdate(dto);

        //售后附件
        ObsFileUtil.editFile(dto.getFileList(), dto.getId());

        for (AfterSalesDetail afterSalesDetail : afterSalesDetailList) {
            afterSalesDetail.setAfterSalesId(dto.getId());
        }
        afterSalesDetailService.editLinked(afterSalesDetailList, AfterSalesDetail::getAfterSalesId, dto.getId());
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @DSTransactional
    @Override
    public void cancellation(Long id) {
        AfterSales afterSales = getById(id);
        this.update(q -> q
                .eq(AfterSales::getId, id)
                .set(AfterSales::getStatus, FlowStatusEnum1.CANCELLATION.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );

        //销毁审批中的流程
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        flowExampleService.update(q -> q
                .eq(FlowExample::getId, afterSales.getFlowId())
                .in(FlowExample::getStatus, FlowStatusEnum.READY_START.getKey(), FlowStatusEnum.IN_PROGRESS.getKey())
                .set(FlowExample::getStatus, FlowStatusEnum.CANCELLATION.getKey())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
        DynamicDataSourceContextHolder.poll();
    }

}
